@@ -1,5 +1,4 @@
-use super::{sink::MatchesSink, SearchConfig};
-use crate::file_entry::FileEntry;
+use super::{file_entry::FileEntry, sink::MatchesSink, SearchConfig};
 use grep::{
     matcher::LineTerminator,
     regex::RegexMatcherBuilder,
@@ -50,15 +49,18 @@ fn run(path: &Path, config: SearchConfig, tx: mpsc::Sender<Event>) {
         .line_terminator(Some(b'\n'))
         .case_insensitive(config.case_insensitive)
         .case_smart(config.case_smart)
+        .word(config.word_regexp)
         .build(&config.pattern)
         .expect("Cannot build RegexMatcher");
-    let mut builder = WalkBuilder::new(path);
 
+    let mut builder = WalkBuilder::new(path);
     let walk_parallel = builder
         .overrides(config.overrides.clone())
         .types(config.types.clone())
         .hidden(!config.search_hidden)
+        .follow_links(config.follow_links)
         .build_parallel();
+
     walk_parallel.run(move || {
         let tx = tx.clone();
         let matcher = matcher.clone();
