@@ -1,17 +1,15 @@
 use anyhow::Result;
-use args::Args;
-use std::io::Write;
-use ui::{
-    editor::Editor,
-    theme::{dark::Dark, light::Light, Theme, ThemeVariant},
-    App,
+use igrep::{
+    app::App,
+    args::Args,
+    editor::EditorCommand,
+    ig,
+    ui::{
+        context_viewer::ContextViewer,
+        theme::{dark::Dark, light::Light, Theme, ThemeVariant},
+    },
 };
-
-mod args;
-mod file_entry;
-mod grep_match;
-pub mod ig;
-pub mod ui;
+use std::io::Write;
 
 fn main() -> Result<()> {
     let args = Args::parse_cli_and_config_file();
@@ -41,6 +39,8 @@ fn main() -> Result<()> {
         .case_insensitive(args.ignore_case)
         .case_smart(args.smart_case)
         .search_hidden(args.search_hidden)
+        .follow_links(args.follow_links)
+        .word_regexp(args.word_regexp)
         .globs(args.glob)?
         .file_types(args.type_matching, args.type_not)?;
 
@@ -48,7 +48,12 @@ fn main() -> Result<()> {
         ThemeVariant::Light => Box::new(Light),
         ThemeVariant::Dark => Box::new(Dark),
     };
-    let mut app = App::new(search_config, Editor::determine(args.editor.editor)?, theme);
+    let mut app = App::new(
+        search_config,
+        EditorCommand::new(args.editor.custom_command, args.editor.editor)?,
+        ContextViewer::new(args.context_viewer),
+        theme,
+    );
     app.run()?;
 
     Ok(())
