@@ -1,13 +1,14 @@
-use super::app::Application;
 use anyhow::Result;
 use crossterm::event::{poll, read, Event, KeyCode, KeyEvent, KeyModifiers};
 use std::time::Duration;
 
+use crate::app::Application;
+
 #[derive(Default)]
 pub struct InputHandler {
     input_buffer: String,
     input_state: InputState,
-    text_insertion: bool,
+    input_mode: InputMode,
 }
 
 #[derive(Clone, Debug, Default, PartialEq, Eq)]
@@ -18,6 +19,14 @@ pub enum InputState {
     Invalid(String),
 }
 
+#[derive(Clone, Debug, Default, PartialEq, Eq)]
+pub enum InputMode {
+    #[default]
+    Normal,
+    TextInsertion,
+    Keymap,
+}
+
 impl InputHandler {
     pub fn handle_input<A: Application>(&mut self, app: &mut A) -> Result<()> {
         let poll_timeout = if app.is_searching() {
@@ -29,10 +38,12 @@ impl InputHandler {
         if poll(poll_timeout)? {
             let read_event = read()?;
             if let Event::Key(key_event) = read_event {
-                if !self.text_insertion {
-                    self.handle_key_in_normal_mode(key_event, app);
-                } else {
-                    self.handle_key_in_text_insertion_mode(key_event, app);
+                match self.input_mode {
+                    InputMode::Normal => self.handle_key_in_normal_mode(key_event, app),
+                    InputMode::TextInsertion => {
+                        self.handle_key_in_text_insertion_mode(key_event, app)
+                    }
+                    InputMode::Keymap => self.handle_key_in_keymap_mode(key_event, app),
                 }
             }
         }
@@ -45,6 +56,7 @@ impl InputHandler {
             KeyEvent {
                 code: KeyCode::Char('c'),
                 modifiers: KeyModifiers::CONTROL,
+                ..
             } => app.on_exit(),
             KeyEvent {
                 code: KeyCode::Char(character),
@@ -66,17 +78,19 @@ impl InputHandler {
             | KeyEvent {
                 code: KeyCode::Char('c'),
                 modifiers: KeyModifiers::CONTROL,
+                ..
             }
             | KeyEvent {
                 code: KeyCode::F(5),
                 ..
             } => {
-                self.text_insertion = false;
+                self.input_mode = InputMode::Normal;
                 app.on_toggle_popup();
             }
             KeyEvent {
                 code: KeyCode::Char(c),
                 modifiers: modifier,
+                ..
             } => {
                 if modifier == KeyModifiers::SHIFT {
                     app.on_char_inserted(c.to_ascii_uppercase());
@@ -92,7 +106,7 @@ impl InputHandler {
                 code: KeyCode::Enter,
                 ..
             } => {
-                self.text_insertion = false;
+                self.input_mode = InputMode::Normal;
                 app.on_search();
                 app.on_toggle_popup();
             }
@@ -100,6 +114,46 @@ impl InputHandler {
         }
     }
 
+    fn handle_key_in_keymap_mode<A: Application>(&mut self, key_event: KeyEvent, app: &mut A) {
+        match key_event {
+            KeyEvent {
+                code: KeyCode::Up, ..
+            }
+            | KeyEvent {
+                code: KeyCode::Char('k'),
+                ..
+            } => app.on_keymap_up(),
+            KeyEvent {
+                code: KeyCode::Down,
+                ..
+            }
+            | KeyEvent {
+                code: KeyCode::Char('j'),
+                ..
+            } => app.on_keymap_down(),
+            KeyEvent {
+                code: KeyCode::Left,
+                ..
+            }
+            | KeyEvent {
+                code: KeyCode::Char('h'),
+                ..
+            } => app.on_keymap_left(),
+            KeyEvent {
+                code: KeyCode::Right,
+                ..
+            }
+            | KeyEvent {
+                code: KeyCode::Char('l'),
+                ..
+            } => app.on_keymap_right(),
+            _ => {
+                self.input_mode = InputMode::Normal;
+                app.on_toggle_keymap();
+            }
+        }
+    }
+
     fn handle_char_input<A: Application>(&mut self, character: char, app: &mut A) {
         self.input_buffer.push(character);
         self.input_state = InputState::Valid;
@@ -132,7 +186,16 @@ impl InputHandler {
             "s" => consume_buffer_and_execute(&mut self.input_buffer, &mut || {
                 app.on_toggle_context_viewer_horizontal()
             }),
+            "+" => consume_buffer_and_execute(&mut self.input_buffer, &mut || {
+                app.on_increase_context_viewer_size()
+            }),
+            "-" => consume_buffer_and_execute(&mut self.input_buffer, &mut || {
+                app.on_decrease_context_viewer_size()
+            }),
             "q" => consume_buffer_and_execute(&mut self.input_buffer, &mut || app.on_exit()),
+            "?" => {
+                consume_buffer_and_execute(&mut self.input_buffer, &mut || app.on_toggle_keymap())
+            }
             "g" => self.input_state = InputState::Incomplete("g…".into()),
             "d" => self.input_state = InputState::Incomplete("d…".into()),
             buf => {
@@ -154,8 +217,12 @@ impl InputHandler {
             KeyCode::End => app.on_bottom(),
             KeyCode::Delete => app.on_remove_current_entry(),
             KeyCode::Enter => app.on_open_file(),
+            KeyCode::F(1) => {
+                self.input_mode = InputMode::Keymap;
+                app.on_toggle_keymap();
+            }
             KeyCode::F(5) => {
-                self.text_insertion = true;
+                self.input_mode = InputMode::TextInsertion;
                 app.on_toggle_popup();
             }
             KeyCode::Esc => {
@@ -178,8 +245,9 @@ impl InputHandler {
 
 #[cfg(test)]
 mod tests {
+    use crate::app::MockApplication;
+
     use super::*;
-    use crate::ui::app::MockApplication;
     use crossterm::event::KeyCode::{Char, Esc};
     use test_case::test_case;
 
@@ -202,6 +270,14 @@ mod tests {
         }
     }
 
+    fn handle_key_keymap_mode<A: Application>(key_event: KeyEvent, app: &mut A) {
+        let mut input_handler = InputHandler {
+            input_mode: InputMode::Keymap,
+            ..Default::default()
+        };
+        input_handler.handle_key_in_keymap_mode(key_event, app);
+    }
+
     #[test_case(KeyCode::Down; "down")]
     #[test_case(Char('j'); "j")]
     fn next_match(key_code: KeyCode) {
@@ -309,6 +385,57 @@ mod tests {
         handle_key(KeyCode::F(5), &mut app_mock);
     }
 
+    #[test_case(KeyCode::F(1))]
+    #[test_case(KeyCode::Char('?'))]
+    fn keymap_open(key_code: KeyCode) {
+        let mut app_mock = MockApplication::default();
+        app_mock.expect_on_toggle_keymap().once().return_const(());
+        handle_key(key_code, &mut app_mock);
+    }
+
+    #[test_case(KeyEvent::new(KeyCode::F(1), KeyModifiers::NONE))]
+    #[test_case(KeyEvent::new(KeyCode::Char('?'), KeyModifiers::NONE))]
+    #[test_case(KeyEvent::new(KeyCode::Esc, KeyModifiers::NONE))]
+    #[test_case(KeyEvent::new(KeyCode::Enter, KeyModifiers::NONE))]
+    #[test_case(KeyEvent::new(KeyCode::Char('c'), KeyModifiers::CONTROL))]
+    fn keymap_close(event: KeyEvent) {
+        let mut app_mock = MockApplication::default();
+        app_mock.expect_on_toggle_keymap().once().return_const(());
+        handle_key_keymap_mode(event, &mut app_mock);
+    }
+
+    #[test_case(KeyEvent::new(KeyCode::Up, KeyModifiers::NONE))]
+    #[test_case(KeyEvent::new(KeyCode::Char('k'), KeyModifiers::NONE))]
+    fn keymap_up(event: KeyEvent) {
+        let mut app_mock = MockApplication::default();
+        app_mock.expect_on_keymap_up().once().return_const(());
+        handle_key_keymap_mode(event, &mut app_mock);
+    }
+
+    #[test_case(KeyEvent::new(KeyCode::Down, KeyModifiers::NONE))]
+    #[test_case(KeyEvent::new(KeyCode::Char('j'), KeyModifiers::NONE))]
+    fn keymap_down(event: KeyEvent) {
+        let mut app_mock = MockApplication::default();
+        app_mock.expect_on_keymap_down().once().return_const(());
+        handle_key_keymap_mode(event, &mut app_mock);
+    }
+
+    #[test_case(KeyEvent::new(KeyCode::Left, KeyModifiers::NONE))]
+    #[test_case(KeyEvent::new(KeyCode::Char('h'), KeyModifiers::NONE))]
+    fn keymap_left(event: KeyEvent) {
+        let mut app_mock = MockApplication::default();
+        app_mock.expect_on_keymap_left().once().return_const(());
+        handle_key_keymap_mode(event, &mut app_mock);
+    }
+
+    #[test_case(KeyEvent::new(KeyCode::Right, KeyModifiers::NONE))]
+    #[test_case(KeyEvent::new(KeyCode::Char('l'), KeyModifiers::NONE))]
+    fn keymap_right(event: KeyEvent) {
+        let mut app_mock = MockApplication::default();
+        app_mock.expect_on_keymap_right().once().return_const(());
+        handle_key_keymap_mode(event, &mut app_mock);
+    }
+
     #[test_case(&[Char('q')]; "q")]
     #[test_case(&[Esc]; "empty input state")]
     #[test_case(&[Char('a'), Char('b'), Esc]; "invalid input state")]
