@@ -8,7 +8,7 @@ use ratatui::{
 use std::iter::Iterator;
 use unicode_width::UnicodeWidthStr;
 
-#[derive(Default, Debug, Clone)]
+#[derive(Default, Debug, Copy, Clone)]
 pub struct ListState {
     offset: usize,
     selected: Option<usize>,
@@ -21,6 +21,10 @@ impl ListState {
             self.offset = 0;
         }
     }
+
+    pub fn selected(&self) -> Option<usize> {
+        self.selected
+    }
 }
 
 #[derive(Debug, Clone)]
